# DBMidProxyServer(DB代理服务)

## 版本日志
### 1.1
- 支持配置多数据，集成支持数据库：MySQL,PostgreSQL,Oracle,SQLServer
- 支持外挂jdbc驱动包，从而实现对默认数据库之外的数据库访问，已测试达梦DM8
- 新的配置文件格式，更简洁方便
- 新增服务信息等接口，方便集成平台获取代理服务信息
- 支持查询多sql执行（为了支持无offset轮询时，一次查询需要使用两个SQL查询的情况）
- 优化错误返回，更全面的异常捕捉
- 升级依赖jar，包括spring-boot，fastjson等，防止漏洞
- 开启druid的统计功能
- 对集成平台全环境兼容（集成平台分环境发布，会多版本共存，旧版本环境也可正常访问新版本代理服务）
## 框架
- web框架：spring-boot
- 开发工具集：hutool
- 鉴权：spring security
- 数据库：mybatis + druid + hutool-db
### 数据库分页
默认使用的pagehelper，但是发现对oracle的分页支持，无法走到索引。测试后hutool-db的实现可以。
所以默认会使用pagehelper，当使用无offset轮询时，会使用自实现的limit逻辑，对oracle和sqlserver参考hutool-db来实现。
区别如下：
```java
    //pagehelper的处理
    public String getPageSql(String sql, Page page, CacheKey pageKey) {
        StringBuilder sqlBuilder = new StringBuilder(sql.length() + 120);
        sqlBuilder.append("SELECT * FROM ( ");
        sqlBuilder.append(" SELECT TMP_PAGE.*, ROWNUM PAGEHELPER_ROW_ID FROM ( \n");
        sqlBuilder.append(sql);
        sqlBuilder.append("\n ) TMP_PAGE)");
        sqlBuilder.append(" WHERE PAGEHELPER_ROW_ID <= ? AND PAGEHELPER_ROW_ID > ?");
        return sqlBuilder.toString();
    }
    
    //hutool-db的处理
    protected SqlBuilder wrapPageSql(SqlBuilder find, Page page) {
    final int[] startEnd = page.getStartEnd();
        return find
        .insertPreFragment("SELECT * FROM ( SELECT row_.*, rownum rownum_ from ( ")
        .append(" ) row_ where rownum <= ").append(startEnd[1])//
        .append(") table_alias_")//
        .append(" where table_alias_.rownum_ > ").append(startEnd[0]);//
    }
```
## 部署说明
### 配置文件
服务有两种配置。
第一种是spring的配置。
默认启用spring的prod环境，所以可以在jar包同级目录创建`application-prod.yml`增加配置。
示例：

```yaml
# application-prod.yml
setting:
  # 设置文件目录 可以使用相对路径,不配置时，默认使用工作目录下的setting文件夹
  dir: D:\project\DBMidProxyServer\setting
server:
  servlet:
    context-path: /dev
```

另外一种是使用hutool的配置。其中user.setting代码已经支持了动态加载。
db.setting未支持。
以上setting.dir指定的目录下必须包含两个配置文件`db.setting`和`user.setting`

```
#db.setting 数据库配置，
#使用druid的配置参考https://github.com/alibaba/druid/wiki/DruidDataSource%E9%85%8D%E7%BD%AE%E5%B1%9E%E6%80%A7%E5%88%97%E8%A1%A8
#分组名就是作为数据源名称
[db1]
url=jdbc:postgresql://localhost:5432/db1?ssl=false
username=postgres
password=1234qwer
queryTimeout=30
initialSize=5
maxActive=20
testWhileIdle=true
[db2]
url=jdbc:postgresql://localhost:5432/db2?ssl=false
username=postgres
password=1234qwer
queryTimeout=30
initialSize=5
maxActive=20
testWhileIdle=true
[db3]
# mysql
url=jdbc:mysql://localhost:3306/db3?useSSL=false&characterEncoding=UTF-8&connectionTimeZone=Asia/Shanghai
username=root
password=1234qwer
queryTimeout=30
initialSize=5
maxActive=20
testWhileIdle=true
[db4]
# sqlserver
url=jdbc:sqlserver://;serverName=localhost;databaseName=db4;encrypt=true;trustServerCertificate=true
username=sa
password=8kn0ye66JxO5
queryTimeout=30
initialSize=5
maxActive=20
testWhileIdle=true
[db5]
# oracle
url=jdbc:oracle:thin:@127.0.0.1:1521:XE
username=dbproxy
password=1234qwer
queryTimeout=30
initialSize=5
maxActive=20
testWhileIdle=true
[dm8]
# 达梦数据库 需要另外增加驱动启动
url=jdbc:dm://localhost:5236?LobMode=1
username=SYSDBA
password=SYSDBA001
queryTimeout=30
initialSize=5
maxActive=20
testWhileIdle=true
# 未集成驱动的，需要指定jdbc驱动的jar包。
driverJarPath=D:\driver\db-driver\DmJdbcDriver18.jar
```

```
# user.setting
# 安全性配置
userName=admin
password=1234qwer
```

### 运行命令
``` shell
java -jar DBMidProxyServer.jar
```

- MySQL
- PostgreSQL
- Oracle
- SQL Server
